﻿"use strict";
const tslib_1 = require("tslib");
const common_1 = require("@kitajs/common");
const parser_1 = require("@kitajs/parser");
const path_1 = tslib_1.__importDefault(require("path"));
const tsserverlibrary_1 = tslib_1.__importDefault(require("typescript/lib/tsserverlibrary"));
const provider_1 = require("./parsers/provider");
const route_1 = require("./parsers/route");
const proxy_1 = require("./util/proxy");
module.exports = function initHtmlPlugin() {
    return {
        create(info) {
            const proxy = (0, proxy_1.proxyObject)(info.languageService);
            let config;
            let providerPaths;
            let parser;
            let rootProvider;
            let rootRoute;
            info.project.projectService.logger.info(`[kita-plugin] Started plugin for ${info.project.getProjectName()} at ${info.project.getCurrentDirectory()}`);
            proxy.getSemanticDiagnostics = function clonedSemanticDiagnostics(filename) {
                // ts typescript will return the path with / on windows
                filename = path_1.default.normalize(filename);
                const diagnostics = info.languageService.getSemanticDiagnostics(filename);
                const start = performance.now();
                const program = info.languageService.getProgram();
                const source = program.getSourceFile(filename);
                if (!source) {
                    return diagnostics;
                }
                try {
                    // Lazy load the parser
                    if (!config) {
                        loadParser(program, start);
                    }
                    // Only parse files inside the cwd
                    if (!filename.startsWith(config.cwd)) {
                        return diagnostics;
                    }
                    // Searches if this file is a provider
                    if (filename.startsWith(rootProvider)) {
                        info.project.projectService.logger.info(`[kita-plugin] Parsed provider ${filename} in ${performance.now() - start}ms`);
                        (0, provider_1.appendProviderDiagnostics)(parser, source, diagnostics);
                        return diagnostics;
                    }
                    // Searches if this file is a route
                    if (filename.startsWith(rootRoute)) {
                        info.project.projectService.logger.info(`[kita-plugin] Parsed route ${filename} in ${performance.now() - start}ms`);
                        (0, route_1.appendRouteDiagnostics)(parser, program, source, diagnostics, providerPaths);
                        return diagnostics;
                    }
                }
                catch (error) {
                    if (error instanceof common_1.KitaError) {
                        diagnostics.push(error.diagnostic);
                        return diagnostics;
                    }
                    info.project.projectService.logger.msg(`[kita-plugin] Error:\n${error.stack}`, tsserverlibrary_1.default.server.Msg.Err);
                }
                return diagnostics;
            };
            function loadParser(program, start) {
                // ts typescript will return the path with / on windows
                const root = path_1.default.normalize(program.getCurrentDirectory());
                try {
                    config = (0, common_1.importConfig)(path_1.default.join(root, 'kita.config.js'));
                }
                catch {
                    config = (0, common_1.parseConfig)({}, root);
                }
                // process.cwd() will return the editor cwd, and not the project cwd
                if (config.cwd === process.cwd()) {
                    config.cwd = root;
                }
                rootRoute = path_1.default.resolve(config.src, common_1.kRoutesFolder);
                rootProvider = path_1.default.resolve(config.src, common_1.kProvidersFolder);
                providerPaths = program
                    .getSourceFiles()
                    .map((f) => f.fileName)
                    .filter((f) => f.startsWith((0, parser_1.toTsPath)(rootProvider)));
                parser = new parser_1.KitaParser(config, program);
                info.project.projectService.logger.info(`[kita-plugin] Successfully parsed config in ${performance.now() - start}ms`);
                info.project.projectService.logger.info(`[kita-plugin] ${JSON.stringify(config, null, 2)}`);
            }
            return proxy;
        }
    };
};
//# sourceMappingURL=index.js.map