﻿"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.appendProviderDiagnostics = void 0;
const common_1 = require("@kitajs/common");
const sync_1 = require("../util/sync");
/** Manual and faster parsing of a single provider file. */
function appendProviderDiagnostics(parser, provider, diagnostics) {
    let supports = parser.rootProviderParser.supports(provider);
    if ((0, common_1.isPromiseLike)(supports)) {
        supports = (0, sync_1.awaitSync)(supports);
    }
    // This should never happens as there is a catch-all in the parser
    if (!supports) {
        diagnostics.push(new common_1.ProviderResolverNotFound(provider).diagnostic);
        return;
    }
    try {
        let parsed = parser.rootProviderParser.parse(provider);
        if ((0, common_1.isPromiseLike)(parsed)) {
            parsed = (0, sync_1.awaitSync)(parsed);
        }
        // success!
    }
    catch (error) {
        if (error instanceof common_1.KitaError) {
            diagnostics.push(error.diagnostic);
        }
        else if (error instanceof Error) {
            diagnostics.push(new common_1.UnknownKitaError(error.message, error).diagnostic);
        }
        else {
            diagnostics.push(new common_1.UnknownKitaError('Unknown error!', error).diagnostic);
        }
    }
}
exports.appendProviderDiagnostics = appendProviderDiagnostics;
//# sourceMappingURL=provider.js.map